/*******************************************************************************

	Project        : PCI express

	Component      : sample app

	Module         : dma.c

	Function       : dma sample

-------------------------------------------------------------------------------
	[history]
	09-27-2010	Ver 1.00   M.Meguro    create
	07-09-2015	Ver	1.03   Y.Watanabe  device number is input by a command line argument

******************************************************************************/
//*******************************************/
//*    include								*/
//*******************************************/
#include	<stdlib.h>
#include	"pcielib.h"

int
main(
	int		argc,
	char	*argv[]
)
{
	HANDLE	hDevice;
	int		ret,dmadoneret=0;
	int		devnum;
	char	*stopstring;
	void	*phy,*usr;
	DWORD	value;
#ifdef	USE_INTR
	HANDLE	hEvent;
#endif

//	Pcie_SetDeviceAndEventName("ABCPcie","Global\\ABCIntrEvent");
	if(argc!=2)
	{
		printf("Usage:\n");
		printf("\tdma [device number]\n");
		printf("\tdevice number:device number functions\n");
		exit(0);
	}
	devnum=strtol(argv[1],&stopstring,0);
	hDevice=Pcie_OpenDevice(devnum);
	if(hDevice!=INVALID_HANDLE_VALUE)
	{
		ret=Pcie_GetDMAMem(hDevice,4*1024*1024,&phy,&usr);
		if(ret!=PCIE_NORMAL)
		{
			printf("Pcie_GetDMAMem error=%d\n",ret);
			goto EXIT;
		}
		printf("%x\n",*(int*)usr);
		Pcie_WriteRegister32(hDevice,0,0x18,(DWORD)phy);
		Pcie_WriteRegister32(hDevice,0,0x1C,(DWORD)0x80000000);
		Pcie_WriteRegister32(hDevice,0,0x20,(DWORD)4*1024*1024);
		Pcie_WriteRegister32(hDevice,0,0x14,(DWORD)0x80);
#ifdef	USE_INTR
		ret=Pcie_RegisterIntr(hDevice,0,0x00,0,0x00,0,0x04,0,0x10,&hEvent);
		if(ret!=PCIE_NORMAL)
		{
			printf("Pcie_RegisterIntr error=%d\n",ret);
			goto EXIT;
		}
		Pcie_WriteRegister32(hDevice,0,0x00,0x1001);	// start Read DMA with INTR
#else
		Pcie_WriteRegister32(hDevice,0,0x00,0x0001);	// start Read DMA
#endif

#ifdef	USE_INTR

		dmadoneret=Pcie_IsDMADone(hDevice,hEvent,INFINITE);
		if(dmadoneret!=PCIE_NORMAL)
		{
			if(dmadoneret==PCIE_ERR_PARAM)
			{
				printf("parameter error=%d\n",dmadoneret);
			}
			else
			{
				printf("do not get dma done signal=%d\n",dmadoneret);
			}
		}
		else
		{
			Pcie_ReadRegister32(hDevice,0,0x04,&value);
			if(value & 0x10)
			{
				Pcie_WriteRegister32(hDevice,0,0x04,0x10);
			}
		}

		ret=Pcie_UnRegisterIntr(hDevice,hEvent);
		if(ret!=PCIE_NORMAL)
		{
			printf("Pcie_UnRegisterIntr error=%d\n",ret);
			goto EXIT;
		}
#else
		for(;;)
		{
			Pcie_ReadRegister32(hDevice,0,0x04,&value);
			if(value & 0x10)
			{
				Pcie_WriteRegister32(hDevice,0,0x04,0x10);
				break;
			}
		}
#endif
		ret=Pcie_RelDMAMem(hDevice);
		if(ret!=PCIE_NORMAL)
		{
			printf("Pcie_RelDMAMem error=%d\n",ret);
			goto EXIT;
		}
		Pcie_WriteRegister32(hDevice,0,0x00,0x00);
		if(dmadoneret==PCIE_NORMAL)
		{
			printf("DMA OK\n");
		}
	}
	else
		printf("driver open error\n");
EXIT:
	if(hDevice!=INVALID_HANDLE_VALUE)
		Pcie_CloseDevice(hDevice);
	return(0);
}
