/*******************************************************************************

	Project        : PCI express

	Component      : sample app

	Module         : readpciconf.c

	Function       : read pci config

-------------------------------------------------------------------------------
	[history]
	09-27-2010	Ver 1.00   M.Meguro    create
	07-09-2015	Ver	1.03   Y.Watanabe  device number is input by a command line argument

******************************************************************************/
//*******************************************/
//*    include								*/
//*******************************************/
#include	"pcielib.h"

BYTE		szBuf[4096];

void
DumpData(
	unsigned char	*value,
	unsigned long	count
)
{
	int		i,j;

	printf("     00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F\r\n");
	printf("---------------------------------------------------------------------\r\n");
	for(i=0;i<(int)(count+15)/16;i++)
	{
		printf("%04X:",i*16);
		for(j=0;j<(int)min(16,count-i*16);j++)
			printf("%02X ",value[i*16+j]&0xff);
		for(;j<16;j++)
			printf("   ");
		for(j=0;j<(int)min(16,count-i*16);j++)
		{
			if((value[i*16+j]&0xff)>=0x20 && (value[i*16+j]&0xff)<=0x7e)
				printf("%c",value[i*16+j]&0xff);
			else
				printf(".");
		}
		printf("\r\n");
	}
}
void
main(
	int		argc,
	char	*argv[]
)
{
	HANDLE	hDevice;
	int		ret;
	int		devnum;
	char	*stopstring;
	DWORD	size;
	DWORD	offset;

	if(argc!=4)
	{
		printf("Usage:\n");
		printf("\treadpciconf [device number] [size] [offset]\n");
		printf("\tdevice number:device number functions\n");
		printf("\tsize:read size[MAX 4096]\n");
		printf("\toffset:configuration area offset\n");
		exit(0);
	}
	devnum=strtol(argv[1],&stopstring,0);
	hDevice=Pcie_OpenDevice(devnum);
	if(hDevice!=INVALID_HANDLE_VALUE)
	{
		size=strtol(argv[2],&stopstring,0);
		offset=strtol(argv[3],&stopstring,0);
		ret=Pcie_ReadPciConfig(hDevice,size,offset,szBuf);
		if(ret!=PCIE_NORMAL)
			printf("Pcie_ReadPciConfig error=%d\n",ret);
		else
			DumpData(szBuf,size);
	}
	else
		printf("driver open error\n");
	if(hDevice!=INVALID_HANDLE_VALUE)
		Pcie_CloseDevice(hDevice);
}
